@@ -27,12 +27,13 @@ import (
 	"github.com/go-kit/kit/log"
 	"github.com/pkg/errors"
 	prom_testutil "github.com/prometheus/client_golang/prometheus/testutil"
+	"github.com/stretchr/testify/require"
+
 	"github.com/prometheus/prometheus/pkg/labels"
 	"github.com/prometheus/prometheus/tsdb/chunkenc"
 	"github.com/prometheus/prometheus/tsdb/chunks"
 	"github.com/prometheus/prometheus/tsdb/fileutil"
 	"github.com/prometheus/prometheus/tsdb/tombstones"
-	"github.com/prometheus/prometheus/util/testutil"
 )
 
 func TestSplitByRange(t *testing.T) {
@@ -134,7 +135,7 @@ func TestSplitByRange(t *testing.T) {
 			}
 		}
 
-		testutil.Equals(t, exp, splitByRange(blocks, c.trange))
+		require.Equal(t, exp, splitByRange(blocks, c.trange))
 	}
 }
 
@@ -158,7 +159,7 @@ func TestNoPanicFor0Tombstones(t *testing.T) {
 	}
 
 	c, err := NewLeveledCompactor(context.Background(), nil, nil, []int64{50}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	c.plan(metas)
 }
@@ -172,7 +173,7 @@ func TestLeveledCompactor_plan(t *testing.T) {
 		540,
 		1620,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	cases := map[string]struct {
 		metas    []dirMeta
@@ -365,8 +366,8 @@ func TestLeveledCompactor_plan(t *testing.T) {
 	for title, c := range cases {
 		if !t.Run(title, func(t *testing.T) {
 			res, err := compactor.plan(c.metas)
-			testutil.Ok(t, err)
-			testutil.Equals(t, c.expected, res)
+			require.NoError(t, err)
+			require.Equal(t, c.expected, res)
 		}) {
 			return
 		}
@@ -381,7 +382,7 @@ func TestRangeWithFailedCompactionWontGetSelected(t *testing.T) {
 		720,
 		2160,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	cases := []struct {
 		metas []dirMeta
@@ -417,9 +418,9 @@ func TestRangeWithFailedCompactionWontGetSelected(t *testing.T) {
 	for _, c := range cases {
 		c.metas[1].meta.Compaction.Failed = true
 		res, err := compactor.plan(c.metas)
-		testutil.Ok(t, err)
+		require.NoError(t, err)
 
-		testutil.Equals(t, []string(nil), res)
+		require.Equal(t, []string(nil), res)
 	}
 }
 
@@ -431,17 +432,17 @@ func TestCompactionFailWillCleanUpTempDir(t *testing.T) {
 		720,
 		2160,
 	}, nil)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 
 	tmpdir, err := ioutil.TempDir("", "test")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdir))
+		require.NoError(t, os.RemoveAll(tmpdir))
 	}()
 
-	testutil.NotOk(t, compactor.write(tmpdir, &BlockMeta{}, erringBReader{}))
+	require.Error(t, compactor.write(tmpdir, &BlockMeta{}, erringBReader{}))
 	_, err = os.Stat(filepath.Join(tmpdir, BlockMeta{}.ULID.String()) + tmpForCreationBlockDirSuffix)
-	testutil.Assert(t, os.IsNotExist(err), "directory is not cleaned up")
+	require.True(t, os.IsNotExist(err), "directory is not cleaned up")
 }
 
 func metaRange(name string, mint, maxt int64, stats *BlockStats) dirMeta {
@@ -461,6 +462,7 @@ func (erringBReader) Index() (IndexReader, error)            { return nil, error
 func (erringBReader) Chunks() (ChunkReader, error)           { return nil, errors.New("chunks") }
 func (erringBReader) Tombstones() (tombstones.Reader, error) { return nil, errors.New("tombstones") }
 func (erringBReader) Meta() BlockMeta                        { return BlockMeta{} }
+func (erringBReader) Size() int64                            { return 0 }
 
 type nopChunkWriter struct{}
 
@@ -939,7 +941,7 @@ func TestCompaction_populateBlock(t *testing.T) {
 			}
 
 			c, err := NewLeveledCompactor(context.Background(), nil, nil, []int64{0}, nil)
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			meta := &BlockMeta{
 				MinTime: tc.compactMinTime,
@@ -952,11 +954,11 @@ func TestCompaction_populateBlock(t *testing.T) {
 			iw := &mockIndexWriter{}
 			err = c.populateBlock(blocks, meta, iw, nopChunkWriter{})
 			if tc.expErr != nil {
-				testutil.NotOk(t, err)
-				testutil.Equals(t, tc.expErr.Error(), err.Error())
+				require.Error(t, err)
+				require.Equal(t, tc.expErr.Error(), err.Error())
 				return
 			}
-			testutil.Ok(t, err)
+			require.NoError(t, err)
 
 			// Check if response is expected and chunk is valid.
 			var raw []seriesSamples
@@ -979,15 +981,15 @@ func TestCompaction_populateBlock(t *testing.T) {
 					}
 
 					// Check if chunk has correct min, max times.
-					testutil.Equals(t, firstTs, chk.MinTime, "chunk Meta %v does not match the first encoded sample timestamp: %v", chk, firstTs)
-					testutil.Equals(t, s.t, chk.MaxTime, "chunk Meta %v does not match the last encoded sample timestamp %v", chk, s.t)
+					require.Equal(t, firstTs, chk.MinTime, "chunk Meta %v does not match the first encoded sample timestamp: %v", chk, firstTs)
+					require.Equal(t, s.t, chk.MaxTime, "chunk Meta %v does not match the last encoded sample timestamp %v", chk, s.t)
 
-					testutil.Ok(t, iter.Err())
+					require.NoError(t, iter.Err())
 					ss.chunks = append(ss.chunks, samples)
 				}
 				raw = append(raw, ss)
 			}
-			testutil.Equals(t, tc.expSeriesSamples, raw)
+			require.Equal(t, tc.expSeriesSamples, raw)
 
 			// Check if stats are calculated properly.
 			s := BlockStats{NumSeries: uint64(len(tc.expSeriesSamples))}
@@ -997,7 +999,7 @@ func TestCompaction_populateBlock(t *testing.T) {
 					s.NumSamples += uint64(len(chk))
 				}
 			}
-			testutil.Equals(t, s, meta.Stats)
+			require.Equal(t, s, meta.Stats)
 		})
 	}
 }
@@ -1047,30 +1049,30 @@ func BenchmarkCompaction(b *testing.B) {
 		nBlocks := len(c.ranges)
 		b.Run(fmt.Sprintf("type=%s,blocks=%d,series=%d,samplesPerSeriesPerBlock=%d", c.compactionType, nBlocks, nSeries, c.ranges[0][1]-c.ranges[0][0]+1), func(b *testing.B) {
 			dir, err := ioutil.TempDir("", "bench_compaction")
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 			defer func() {
-				testutil.Ok(b, os.RemoveAll(dir))
+				require.NoError(b, os.RemoveAll(dir))
 			}()
 			blockDirs := make([]string, 0, len(c.ranges))
 			var blocks []*Block
 			for _, r := range c.ranges {
 				block, err := OpenBlock(nil, createBlock(b, dir, genSeries(nSeries, 10, r[0], r[1])), nil)
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 				blocks = append(blocks, block)
 				defer func() {
-					testutil.Ok(b, block.Close())
+					require.NoError(b, block.Close())
 				}()
 				blockDirs = append(blockDirs, block.Dir())
 			}
 
 			c, err := NewLeveledCompactor(context.Background(), nil, log.NewNopLogger(), []int64{0}, nil)
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 
 			b.ResetTimer()
 			b.ReportAllocs()
 			for i := 0; i < b.N; i++ {
 				_, err = c.Compact(dir, blockDirs, blocks)
-				testutil.Ok(b, err)
+				require.NoError(b, err)
 			}
 		})
 	}
@@ -1078,27 +1080,30 @@ func BenchmarkCompaction(b *testing.B) {
 
 func BenchmarkCompactionFromHead(b *testing.B) {
 	dir, err := ioutil.TempDir("", "bench_compaction_from_head")
-	testutil.Ok(b, err)
+	require.NoError(b, err)
 	defer func() {
-		testutil.Ok(b, os.RemoveAll(dir))
+		require.NoError(b, os.RemoveAll(dir))
 	}()
 	totalSeries := 100000
 	for labelNames := 1; labelNames < totalSeries; labelNames *= 10 {
 		labelValues := totalSeries / labelNames
 		b.Run(fmt.Sprintf("labelnames=%d,labelvalues=%d", labelNames, labelValues), func(b *testing.B) {
 			chunkDir, err := ioutil.TempDir("", "chunk_dir")
-			testutil.Ok(b, err)
+			require.NoError(b, err)
 			defer func() {
-				testutil.Ok(b, os.RemoveAll(chunkDir))
+				require.NoError(b, os.RemoveAll(chunkDir))
 			}()
-			h, err := NewHead(nil, nil, nil, 1000, chunkDir, nil, DefaultStripeSize, nil)
-			testutil.Ok(b, err)
+			opts := DefaultHeadOptions()
+			opts.ChunkRange = 1000
+			opts.ChunkDirRoot = chunkDir
+			h, err := NewHead(nil, nil, nil, opts)
+			require.NoError(b, err)
 			for ln := 0; ln < labelNames; ln++ {
 				app := h.Appender(context.Background())
 				for lv := 0; lv < labelValues; lv++ {
-					app.Add(labels.FromStrings(fmt.Sprintf("%d", ln), fmt.Sprintf("%d%s%d", lv, postingsBenchSuffix, ln)), 0, 0)
+					app.Append(0, labels.FromStrings(fmt.Sprintf("%d", ln), fmt.Sprintf("%d%s%d", lv, postingsBenchSuffix, ln)), 0, 0)
 				}
-				testutil.Ok(b, app.Commit())
+				require.NoError(b, app.Commit())
 			}
 
 			b.ResetTimer()
@@ -1118,7 +1123,7 @@ func BenchmarkCompactionFromHead(b *testing.B) {
 func TestDisableAutoCompactions(t *testing.T) {
 	db := openTestDB(t, nil, nil)
 	defer func() {
-		testutil.Ok(t, db.Close())
+		require.NoError(t, db.Close())
 	}()
 
 	blockRange := db.compactor.(*LeveledCompactor).ranges[0]
@@ -1129,12 +1134,12 @@ func TestDisableAutoCompactions(t *testing.T) {
 	db.DisableCompactions()
 	app := db.Appender(context.Background())
 	for i := int64(0); i < 3; i++ {
-		_, err := app.Add(label, i*blockRange, 0)
-		testutil.Ok(t, err)
-		_, err = app.Add(label, i*blockRange+1000, 0)
-		testutil.Ok(t, err)
+		_, err := app.Append(0, label, i*blockRange, 0)
+		require.NoError(t, err)
+		_, err = app.Append(0, label, i*blockRange+1000, 0)
+		require.NoError(t, err)
 	}
-	testutil.Ok(t, app.Commit())
+	require.NoError(t, app.Commit())
 
 	select {
 	case db.compactc <- struct{}{}:
@@ -1148,8 +1153,8 @@ func TestDisableAutoCompactions(t *testing.T) {
 		time.Sleep(10 * time.Millisecond)
 	}
 
-	testutil.Assert(t, prom_testutil.ToFloat64(db.metrics.compactionsSkipped) > 0.0, "No compaction was skipped after the set timeout.")
-	testutil.Equals(t, 0, len(db.blocks))
+	require.Greater(t, prom_testutil.ToFloat64(db.metrics.compactionsSkipped), 0.0, "No compaction was skipped after the set timeout.")
+	require.Equal(t, 0, len(db.blocks))
 
 	// Enable the compaction, trigger it and check that the block is persisted.
 	db.EnableCompactions()
@@ -1163,16 +1168,16 @@ func TestDisableAutoCompactions(t *testing.T) {
 		}
 		time.Sleep(100 * time.Millisecond)
 	}
-	testutil.Assert(t, len(db.Blocks()) > 0, "No block was persisted after the set timeout.")
+	require.Greater(t, len(db.Blocks()), 0, "No block was persisted after the set timeout.")
 }
 
 // TestCancelCompactions ensures that when the db is closed
 // any running compaction is cancelled to unblock closing the db.
 func TestCancelCompactions(t *testing.T) {
 	tmpdir, err := ioutil.TempDir("", "testCancelCompaction")
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdir))
+		require.NoError(t, os.RemoveAll(tmpdir))
 	}()
 
 	// Create some blocks to fall within the compaction range.
@@ -1183,18 +1188,18 @@ func TestCancelCompactions(t *testing.T) {
 	// Copy the db so we have an exact copy to compare compaction times.
 	tmpdirCopy := tmpdir + "Copy"
 	err = fileutil.CopyDirs(tmpdir, tmpdirCopy)
-	testutil.Ok(t, err)
+	require.NoError(t, err)
 	defer func() {
-		testutil.Ok(t, os.RemoveAll(tmpdirCopy))
+		require.NoError(t, os.RemoveAll(tmpdirCopy))
 	}()
 
 	// Measure the compaction time without interrupting it.
 	var timeCompactionUninterrupted time.Duration
 	{
 		db, err := open(tmpdir, log.NewNopLogger(), nil, DefaultOptions(), []int64{1, 2000})
-		testutil.Ok(t, err)
-		testutil.Equals(t, 3, len(db.Blocks()), "initial block count mismatch")
-		testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
+		require.NoError(t, err)
+		require.Equal(t, 3, len(db.Blocks()), "initial block count mismatch")
+		require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
 		db.compactc <- struct{}{} // Trigger a compaction.
 		var start time.Time
 		for prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.populatingBlocks) <= 0 {
@@ -1207,14 +1212,14 @@ func TestCancelCompactions(t *testing.T) {
 		}
 		timeCompactionUninterrupted = time.Since(start)
 
-		testutil.Ok(t, db.Close())
+		require.NoError(t, db.Close())
 	}
 	// Measure the compaction time when closing the db in the middle of compaction.
 	{
 		db, err := open(tmpdirCopy, log.NewNopLogger(), nil, DefaultOptions(), []int64{1, 2000})
-		testutil.Ok(t, err)
-		testutil.Equals(t, 3, len(db.Blocks()), "initial block count mismatch")
-		testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
+		require.NoError(t, err)
+		require.Equal(t, 3, len(db.Blocks()), "initial block count mismatch")
+		require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial compaction counter mismatch")
 		db.compactc <- struct{}{} // Trigger a compaction.
 		dbClosed := make(chan struct{})
 
@@ -1222,7 +1227,7 @@ func TestCancelCompactions(t *testing.T) {
 			time.Sleep(3 * time.Millisecond)
 		}
 		go func() {
-			testutil.Ok(t, db.Close())
+			require.NoError(t, db.Close())
 			close(dbClosed)
 		}()
 
@@ -1230,11 +1235,11 @@ func TestCancelCompactions(t *testing.T) {
 		<-dbClosed
 		actT := time.Since(start)
 		expT := time.Duration(timeCompactionUninterrupted / 2) // Closing the db in the middle of compaction should less than half the time.
-		testutil.Assert(t, actT < expT, "closing the db took more than expected. exp: <%v, act: %v", expT, actT)
+		require.True(t, actT < expT, "closing the db took more than expected. exp: <%v, act: %v", expT, actT)
 	}
 }
 
-// TestDeleteCompactionBlockAfterFailedReload ensures that a failed reload immediately after a compaction
+// TestDeleteCompactionBlockAfterFailedReload ensures that a failed reloadBlocks immediately after a compaction
 // deletes the resulting block to avoid creatings blocks with the same time range.
 func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 	tests := map[string]func(*DB) int{
@@ -1244,13 +1249,13 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 
 			// Add some data to the head that is enough to trigger a compaction.
 			app := db.Appender(context.Background())
-			_, err := app.Add(defaultLabel, 1, 0)
-			testutil.Ok(t, err)
-			_, err = app.Add(defaultLabel, 2, 0)
-			testutil.Ok(t, err)
-			_, err = app.Add(defaultLabel, 3+rangeToTriggerCompaction, 0)
-			testutil.Ok(t, err)
-			testutil.Ok(t, app.Commit())
+			_, err := app.Append(0, defaultLabel, 1, 0)
+			require.NoError(t, err)
+			_, err = app.Append(0, defaultLabel, 2, 0)
+			require.NoError(t, err)
+			_, err = app.Append(0, defaultLabel, 3+rangeToTriggerCompaction, 0)
+			require.NoError(t, err)
+			require.NoError(t, app.Commit())
 
 			return 0
 		},
@@ -1263,8 +1268,8 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 			for _, m := range blocks {
 				createBlock(t, db.Dir(), genSeries(1, 1, m.MinTime, m.MaxTime))
 			}
-			testutil.Ok(t, db.reload())
-			testutil.Equals(t, len(blocks), len(db.Blocks()), "unexpected block count after a reload")
+			require.NoError(t, db.reload())
+			require.Equal(t, len(blocks), len(db.Blocks()), "unexpected block count after a reloadBlocks")
 
 			return len(blocks)
 		},
@@ -1274,35 +1279,35 @@ func TestDeleteCompactionBlockAfterFailedReload(t *testing.T) {
 		t.Run(title, func(t *testing.T) {
 			db := openTestDB(t, nil, []int64{1, 100})
 			defer func() {
-				testutil.Ok(t, db.Close())
+				require.NoError(t, db.Close())
 			}()
 			db.DisableCompactions()
 
 			expBlocks := bootStrap(db)
 
-			// Create a block that will trigger the reload to fail.
+			// Create a block that will trigger the reloadBlocks to fail.
 			blockPath := createBlock(t, db.Dir(), genSeries(1, 1, 200, 300))
 			lastBlockIndex := path.Join(blockPath, indexFilename)
 			actBlocks, err := blockDirs(db.Dir())
-			testutil.Ok(t, err)
-			testutil.Equals(t, expBlocks, len(actBlocks)-1) // -1 to exclude the corrupted block.
-			testutil.Ok(t, os.RemoveAll(lastBlockIndex))    // Corrupt the block by removing the index file.
+			require.NoError(t, err)
+			require.Equal(t, expBlocks, len(actBlocks)-1)    // -1 to exclude the corrupted block.
+			require.NoError(t, os.RemoveAll(lastBlockIndex)) // Corrupt the block by removing the index file.
 
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "initial 'failed db reload' count metrics mismatch")
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial `compactions` count metric mismatch")
-			testutil.Equals(t, 0.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "initial `compactions failed` count metric mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "initial 'failed db reloadBlocks' count metrics mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "initial `compactions` count metric mismatch")
+			require.Equal(t, 0.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "initial `compactions failed` count metric mismatch")
 
 			// Do the compaction and check the metrics.
-			// Compaction should succeed, but the reload should fail and
+			// Compaction should succeed, but the reloadBlocks should fail and
 			// the new block created from the compaction should be deleted.
-			testutil.NotOk(t, db.Compact())
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "'failed db reload' count metrics mismatch")
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "`compaction` count metric mismatch")
-			testutil.Equals(t, 1.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "`compactions failed` count metric mismatch")
+			require.Error(t, db.Compact())
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.metrics.reloadsFailed), "'failed db reloadBlocks' count metrics mismatch")
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.compactor.(*LeveledCompactor).metrics.ran), "`compaction` count metric mismatch")
+			require.Equal(t, 1.0, prom_testutil.ToFloat64(db.metrics.compactionsFailed), "`compactions failed` count metric mismatch")
 
 			actBlocks, err = blockDirs(db.Dir())
-			testutil.Ok(t, err)
-			testutil.Equals(t, expBlocks, len(actBlocks)-1, "block count should be the same as before the compaction") // -1 to exclude the corrupted block.
+			require.NoError(t, err)
+			require.Equal(t, expBlocks, len(actBlocks)-1, "block count should be the same as before the compaction") // -1 to exclude the corrupted block.
 		})
 	}
 }
